/*
 * War.java
 * Created on February 13, 2006, 5:04 PM
 */

import java.util.Random;

/**
 * War is the main game logic class for playing the card game 'war' and defines all
 * the basic rules and interactions for the game.
 * @author Jheinka
 */
public class War {

    /**
     * Reference to the War view.
     */
    private WarGUI gui;
    
    /**
     * The current round number
     */
    private int round;

    /**
     * User of the program that is currently playing the card game.
     */
    private Player humanPlayer = null;
    /**
     * Artificially simulated opponent to play against the human player.
     */
    private Player computerPlayer = null;
    /**
     * Generates pseudo-random numbers for the computer player to use as a basis for
     * decision making.
     */
    private Random randomNumberGenerator = new Random();
    
    /**
     * The randomly selected index of the card the computer will play
     */
    private int computerCardSelectedIndex;

    /**
     * Debugging Variable
     */
    private boolean debug = false;

    /**
     * Returns the human player of the game.
     * 
     * @return human player of the war game.
     */
    public Player getHumanPlayer() {
        return humanPlayer;
    }

    /**
     * Returns the computer player of the game.
     * 
     * @return computer player of the war game.
     */
    public Player getComputerPlayer() {
        return computerPlayer;
    }
    
    /**
     * Returns the selected computer card's index.
     * 
     * @return the index of the computer card that was selected
     */
	public int getComputerCardSelectedIndex() {
		return computerCardSelectedIndex;
	}
	
	
    /**
     * Creates a new instance of War.
     * 
     * @param playerName name of the human player that is going to be playing the
     *                   card game.
     */
    public War(String playerName, WarGUI g) {
    	round = 0;
        gui = g;
        humanPlayer = new Player(playerName);
        computerPlayer = new Player("Computer");
        CardPile startingDeck = createDeck();
        dealCards(startingDeck);
    }

    /**
      * Sets up the deck for the game and returns it as a <code>CardPile</code>.
      * 
      * @return the deck of cards that will be used for playing war.
      */
     private CardPile createDeck() {
         CardPile deck = new CardPile();

         for (int rank = 2; rank <= 14; rank++) {
             deck.addCard(new Card(rank, "Hearts"));
             deck.addCard(new Card(rank, "Diamonds"));
             deck.addCard(new Card(rank, "Clubs"));
             deck.addCard(new Card(rank, "Spades"));
         }

         return deck;
     }

    /**
      * Shuffles the cards in the deck and distributes them appropriately
      * to the players.
      * 
      * @param deck the deck of cards to be dealt evenly to the players.
      */
     private void dealCards(CardPile deck) {
         deck.shuffle();
         while (deck.cardCount() >= 2) {
             humanPlayer.getDrawPile().addCard(deck.removeTopCard());
             computerPlayer.getDrawPile().addCard(deck.removeTopCard());
         }
         humanPlayer.refillHand();
         computerPlayer.refillHand();
         if (debug) {
             System.out.println("P: " + humanPlayer.getHand().cardCount());
             System.out.println("C: " + computerPlayer.getHand().cardCount());
         }
         gui.setHumanPlayer(humanPlayer);
         gui.setComputerPlayer(computerPlayer);
         gui.updatePiles();
     }

    /**
     * Selects a card from the player's current pile of playable cards.
     * 
     * @param index the index number of the card that was selected from the 
     *              player's playable hand.
     */
    public Card selectPlayerCard(int index) {
        Card playerCard = humanPlayer.getHand().removeCardAt(index);
        humanPlayer.refillHand();
        //gui.updatePiles();
        playerCard.setFaceUp(true);
        if (humanPlayer.getDrawPile().cardCount() == 0) {
        	round++;
        	if (humanPlayer.getWonPile().cardCount() == 0) {
        	    checkForGameOver();
        	} else {
        	    //shuffle won pile and transfer to draw pile
        	   humanPlayer.refillDrawPile();
        	   gui.updatePiles();
        	}
        } 
        return playerCard;
    }
    


    /**
     * Randomly selects a card from the computer's pile of playable cards.
     * @return the card to be played
     */
    public Card getComputerCardSelection() {
        int randomSelectedValue = randomNumberGenerator.nextInt(5);
        Card computerCard = computerPlayer.getHand().removeCardAt(randomSelectedValue);
        computerCardSelectedIndex = randomSelectedValue;
        computerCard.setFaceUp(true);
        computerPlayer.refillHand();
        if (computerPlayer.getDrawPile().cardCount() == 0) {
            if (computerPlayer.getWonPile().cardCount() == 0) {
                checkForGameOver();
            } else {
                //shuffle won pile and transfer to draw pile
                computerPlayer.refillDrawPile();
                gui.updatePiles();
            }
        }
        gui.updatePiles();
        return computerCard;
    }

    /**
     * Initiates a card battle between the human selected card and the computer
     * selected card.
     * @param playerCard the card previously selected by the human player
     * @param computerCard the card selected by the computer player
     */
    public void cardBattle(Card playerCard, Card computerCard) {   
    	
        playerCard.setFaceUp(true);
        computerCard.setFaceUp(true);

        humanPlayer.getGamblePile().addCard(playerCard);
        computerPlayer.getGamblePile().addCard(computerCard);
        gui.updateGamblePiles(humanPlayer, computerPlayer);
        compareCards(playerCard, computerCard);
    }

    /**
     * Compares the card the player selected with the card the computer selected to
     * determine a winner.
     * 
     * @param playerCard card that the player selected.
     * @param computerCard card selected for the computer.
     */
    private void compareCards(Card playerCard, Card computerCard) {
        if (playerCard.isGreaterThan(computerCard)) {
            playerWinsBattle();
            //used for the animations in the GUI
            	gui.setIsWar(false);
            	gui.isPlayerWinner(true);
            	gui.setHumanPlayer(humanPlayer);
            	gui.setComputerPlayer(computerPlayer);
        } else if (computerCard.isGreaterThan(playerCard)) {
            computerWinsBattle();
            //used for the animations in the GUI
            	gui.setIsWar(false);
            	gui.isPlayerWinner(false);
            	gui.setHumanPlayer(humanPlayer);
            	gui.setComputerPlayer(computerPlayer);
        } else {
        	gui.setIsWar(true);
        	gui.setHumanPlayer(humanPlayer);
        	gui.setComputerPlayer(computerPlayer);
        	if (round == 3) {
        	    if (humanPlayer.getDrawPile().cardCount() <= 3) {
        	        if (humanPlayer.getWonPile().cardCount() > computerPlayer.getWonPile().cardCount()) {
        	            gameOver(humanPlayer, "Draw pile was too low");
        	        } else {
        	            gameOver(computerPlayer, "Draw pile was too low");
        	        }
        	    }
        	}
        	gui.war();
            
        	cardWar();            
        }
        checkForGameOver();
    }

    /**
     * Increases the number of cards being wagered on the upcoming battle and 
     * returns to waiting for the player to select a new card to decide the 
     * victory of the next battle with the higher stakes.  Called in case of a
     * war scenario (selected cards have the same rank).
     */
    private void cardWar() {
        	increaseGamble(humanPlayer);
        	increaseGamble(computerPlayer);
    }

    /**
     * Takes three cards from the given player's hand and adds them automatically to
     * the player's gamble pile and refills the player's hand.
     * 
     * @param player the current player to increase the gamble for.
     */
    private void increaseGamble(Player player) {
        boolean gameOver = false;
        for (int i = 0; i < 3; i++) {
            if (player.getHand().cardCount() > 1) {
                Card gambleCard = player.getHand().removeTopCard();
                gambleCard.setFaceUp(false);
                player.getGamblePile().addCard(gambleCard);
            }
        }
        if (player.getDrawPile().cardCount() <= 4) {
            if (player.getWonPile().cardCount() >= 4) {
                player.refillDrawPile();
            } else {
                gameOver = checkForGameOver();
            }
        }
        if (!gameOver) {
            player.refillHand();
        }
    }
    

    /**
     * Collects the cards from both gamble piles and adds them  to the human's win
     * pile.  Called when the human player wins the hand.
     */
    private void playerWinsBattle() {
        while (humanPlayer.getGamblePile().cardCount() > 0) {
            Card wonCard = humanPlayer.getGamblePile().removeTopCard();
            humanPlayer.getWonPile().addCard(wonCard);
        }
        while (computerPlayer.getGamblePile().cardCount() > 0) {
            Card wonCard = computerPlayer.getGamblePile().removeTopCard();
            humanPlayer.getWonPile().addCard(wonCard);
        }
        //if (round != 3) {
        //	humanPlayer.refillHand();
        //	computerPlayer.refillHand();
        //}
    }

    /**
     * Collects the cards from both gamble piles and adds them to the computer's
     * win pile.  Called when the computer player wins the hand.
     */
    private void computerWinsBattle() {
        while (humanPlayer.getGamblePile().cardCount() > 0) {
            Card wonCard = humanPlayer.getGamblePile().removeTopCard();
            computerPlayer.getWonPile().addCard(wonCard);
        }
        while (computerPlayer.getGamblePile().cardCount() > 0) {
            Card wonCard = computerPlayer.getGamblePile().removeTopCard();
            computerPlayer.getWonPile().addCard(wonCard);
        }
        //if (round != 3) {
        //	humanPlayer.refillHand();
        //	computerPlayer.refillHand();
        //}
    }

    /**
     * Determines whether or not any of the end game conditions have been met.
     */
    private boolean checkForGameOver() {
        String message = "";
    	boolean gameOver = false;
        if (humanPlayer.getHand().cardCount() < 5) {
        	gameOver = true;
        	message = "You do not have enough cards left";
            gameOver(computerPlayer, message);
        } else if (computerPlayer.getHand().cardCount() < 5) {
            message = "The computer ran out of cards";
            gameOver = true;
            gameOver(humanPlayer, message);
        } else if (round == 3) {
            int computerTotalCards = computerPlayer.getWonPile().cardCount() + 
            					 computerPlayer.getDrawPile().cardCount();
            int playerTotalCards = humanPlayer.getWonPile().cardCount() +
            				   humanPlayer.getDrawPile().cardCount();
        	if (computerTotalCards > playerTotalCards) {
        		//computer player won
        	    message = "The computer has the most cards";
        	    gameOver = true;
        		gameOver(computerPlayer, message);
        		
        	} else if (playerTotalCards > computerTotalCards) {
        		//human player won
        	    gameOver = true;
        	    message = "The player has the most cards";
        		gameOver(humanPlayer, message);
        	} else if (playerTotalCards == computerTotalCards) {
        		//tie game
        		gameOver = true;
        		message = "This was a tie game";
        		gameOver(humanPlayer, message);
        	}
        	
        }
        return gameOver;
    }

    /**
     * Ends the game, handles all the end game logic, and updates the user 
     * interface with the appropriate winner information.
     * 
     * @param winner the winner of the game.
     */
    private void gameOver(Player winner, String message) {
        gui.gameOver(winner, message);
    }
}
