
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Class that displayed a picture with a caption in a JPanel
 * @author Barb Ericson
 * Copyright 2004 Georgia Institute of Technology
 */
public class PicturePanel extends JPanel
{
  
  ///////////////////////// fields /////////////////////////
  private PictureInfo pictureInfo;
  private boolean editable = true;
  private int desiredHeight = 100;
  private Font font = new Font("Helvetica",Font.PLAIN,12);
  private JLabel titleLabel = new JLabel();
  private JLabel pictureLabel = new JLabel();
  private JTextArea captionArea = new JTextArea("",3,20);
  private JScrollPane scrollPane = null;
  
  ///////////////////////// constructors ///////////////////
  
  /**
   * Constructor that takes no arguments - all fields
   * will be their default values
   */
  public PicturePanel() {}
  
  /**
   * Constructor that takes the file name of the picture file
   * and the caption
   */
  public PicturePanel(PictureInfo info)
  {
    pictureInfo = info;
    
    // initialize the panel
    initPanel();
    
    // reset
    resetPanel();
  }
  
  ////////////////////// methods ///////////////////////////////
  
  /**
   * Method to change the picture info
   */
  public void setPictureInfo(PictureInfo info)
  {
    pictureInfo = info;
    resetPanel();
  }
  
  /**
   * Method to set-up the caption text area
   */
  private void setUpCaptionArea()
  {
    captionArea.setFont(font); // set the font to use for the text
    captionArea.setLineWrap(true); // wrap at the end of the line
    captionArea.setWrapStyleWord(true); // wrap at a word boundary
    captionArea.setMargin(new Insets(1,2,1,1)); // leave 2 pixels on left as margin
    scrollPane = new JScrollPane(captionArea);
    scrollPane.setVerticalScrollBarPolicy(
             JScrollPane.VERTICAL_SCROLLBAR_ALWAYS); // show vertical scroll
    captionArea.setEditable(editable); // set the edit or not flag
    captionArea.addFocusListener(new FocusListener() {
      public void focusLost(FocusEvent e) {
        pictureInfo.setCaption(captionArea.getText());
      }
      public void focusGained(FocusEvent e) {
      }
    });
  }
  
  /**
   * Method to initialize the panel
   */
  private void initPanel()
  {
    
    // set up the title panel
    JPanel titlePanel = new JPanel();
    titlePanel.setLayout(new FlowLayout());
    titlePanel.add(titleLabel);
    
    // set up the picture label
    pictureLabel.setToolTipText("Click on the picture to see the full size picture in another window");
    pictureLabel.addMouseListener(new MyMouseAdapter());
    pictureLabel.setBorder(BorderFactory.createLineBorder(Color.black));
    
    // set up the image panel
    JPanel imagePanel = new JPanel();
    imagePanel.setLayout(new FlowLayout());
    imagePanel.add(pictureLabel);
    
    // set up the caption area
    setUpCaptionArea();
    
     // set-up the panel
    this.setLayout(new BorderLayout());
    this.add(titlePanel,BorderLayout.NORTH);
    this.add(imagePanel,BorderLayout.CENTER);
    this.add(scrollPane,BorderLayout.SOUTH);
    this.setBorder(BorderFactory.createEtchedBorder());
    
  }
  
  /**
   * Method to clear the panel.  It removes everything from the picture label
   * And sets the title label and caption area to the blank string.
   */
  public void clearPanel()
  {
    pictureLabel.setIcon(null);
    titleLabel.setText("No File");
    captionArea.setText("");
  }
  
  /**
   * Method to reset the panel with the picture in a JLabel and
   * the caption in a textarea
   */
  private void resetPanel()
  {
    ImageIcon icon = null;
    int desiredWidth = 20;
    String pictureFile = pictureInfo.getFileName();
    
    // Get the scaled image
    if (pictureFile != null)
    {
      icon = new ImageIcon(pictureFile);
      Image image = icon.getImage();
      desiredWidth = image.getWidth(this);
      
      // if the image is larger than the desired height
      if (desiredHeight < image.getHeight(this))
      {
        // scale to desired height
        desiredWidth = (int) (image.getWidth(this) * 
                              ((float) desiredHeight / image.getHeight(this)));
        image = image.getScaledInstance(desiredWidth,desiredHeight,Image.SCALE_FAST);
        icon.setImage(image);
      }
    }
    
    // set the icon if one or else the text
    if (icon != null)
      pictureLabel.setIcon(icon);
    else
      pictureLabel.setText("No Picture");
   
    // set the title for the title label
    String title = pictureFile;
    int endIndex = Math.max(pictureFile.lastIndexOf("\\"),
                            pictureFile.lastIndexOf("/"));
    if (endIndex >= 0)
      title = pictureFile.substring(endIndex + 1);
    titleLabel.setText(title);
    
    // set the caption text
    captionArea.setText(pictureInfo.getCaption());
    captionArea.setColumns(desiredWidth/10);
    
    // validate the panel to force reset sizes
    this.validate();
    
  }
  
  /**
   * Method to open a main window with the image displayed full size
   */
  public void openDisplayWindow()
  {
    String fileName = pictureInfo.getFileName();
    // Get the image icon
    ImageIcon icon = new ImageIcon(fileName);
    JLabel iconLabel = new JLabel(icon);
    Image image = icon.getImage();
    iconLabel.setToolTipText("Width: " + image.getWidth(this) + 
                             " Height: " + image.getHeight(this));
    JScrollPane scrollPane = new JScrollPane(iconLabel);
    JFrame frame = new JFrame(fileName);
    frame.getContentPane().add(scrollPane);
    frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    frame.pack();
    frame.setVisible(true);
  }
  
  
  /**
   * Private class that extends Mouse Adapter to use to pop-up the full sized 
   * image in a scroll-pane
   */
  private class MyMouseAdapter extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    {
      openDisplayWindow();
    }
  }
  
  /**
   * Method for returning a string about the picture panel
   */
  public String toString()
  {
    return pictureInfo.getFileName();
  }
  
  /**
   * Main method for testing 
   */
  public static void main(String[] argv)
  {
    PicturePanel picturePanel = new PicturePanel(new PictureInfo(Picture.getMediaPath("beach.jpg"),
                                                 "This is a picture of a beach"));
    JFrame frame = new JFrame("test");
    frame.getContentPane().add(picturePanel);
    frame.pack();
    frame.setVisible(true);
  }
 
}